import 'dart:io';

import 'package:globe_cli/src/command_runner.dart';
import 'package:globe_cli/src/exit.dart';
import 'package:globe_cli/src/utils/http_server.dart';
import 'package:mason_logger/mason_logger.dart';
import 'package:pub_updater/pub_updater.dart';

Future<void> main(
  List<String> args, {
  PubUpdater? pubUpdater,
  GlobeHttpServer? httpServer,
}) async {
  await _flushThenExit(
    await GlobeCliCommandRunner(
      pubUpdater: pubUpdater,
      httpServer: httpServer,
    ).run(args),
  );
}

/// Flushes the stdout and stderr streams, then exits the program with the given
/// status code.
///
/// This returns a Future that will never complete, since the program will have
/// exited already. This is useful to prevent Future chains from proceeding
/// after you've decided to exit.
Future<void> _flushThenExit(int status) async {
  try {
    await Future.wait<void>([stdout.close(), stderr.close()]);
    exitOverride(status);
  } on ExitException {
    // Let tests handle this themselves. But we shouldn't have tests consider
    // fake exits as a "software failure".
    rethrow;
  } catch (_) {
    exitOverride(ExitCode.software.code);
  }
}
